local BattleFuncSet = false
freeslot("MT_THEFIST", "S_THEFIST")

mobjinfo[MT_THEFIST] = {
	doomednum = -1,
    spawnstate = S_THEFIST,
	spawnhealth = 100000,
    reactiontime = 1,
    radius = 48*FRACUNIT,
    height = 48*FRACUNIT,
    flags = MF_NOGRAVITY|MF_NOCLIPHEIGHT
}

states[S_THEFIST] = {
	sprite = SPR_FRND,
	frame = E,
	tics = -1,
	nextframe = S_THEFIST,
	var1 = 0,
	var2 = 0
}
mobjinfo[MT_THEFIST].name = "comically large fist"

local function isInBattle(player)
	local battleGT = {GT_ARENA, GT_TEAMARENA, GT_SURVIVAL, GT_TEAMSURVIVAL, GT_BATTLECTF, GT_CP, GT_TEAMCP, GT_DIAMOND, GT_TEAMDIAMOND}
	for _,k in ipairs(battleGT) do
		if (gametype == k) return true end
	end
	return false
end

local function isPlayerVictim(mo)
	if (mo.valid) and (mo.player)
		if (mo.player.powers[pw_carry] == CR_NONE)
		and not (mo.player.spectator)
		and not (mo.player.powers[pw_invulnerability]) 
		and not (mo.player.powers[pw_flashing])
			return true
		else
			return false
		end
	else
		return false
	end
end

local function isEnemyVictim(victim)
	if (victim.valid)
	and ((victim.flags & MF_ENEMY) or (victim.flags & MF_BOSS))
		if (victim.state != mobjinfo[victim.type].painstate)
		and (victim.state != mobjinfo[victim.type].deathstate)
		and (victim.health > 0)
			return true
		else
			return false
		end
	else
		return false
	end
end

local function canBeNerfed(player)
	if (CV_FindVar("freindly_nerf").value == 1)
		if (gametyperules & GTR_RINGSLINGER)
		or (gametyperules & GTR_TAG)
		or (isInBattle(player))
		or (gametyperules & GTR_RACE)
			return true
		else
			return false
		end
	else
		return false
	end
end

addHook("MobjMoveCollide", function(fist, victim)
    if fist.valid and fist.target.valid and fist.target.player and fist.target.skin == "freindlyfellow" and victim.valid
	and (fist.target != victim)
	and (victim.z + victim.height + victim.momz >= fist.z + fist.target.momz)
	and (victim.z + victim.height + victim.momz <= fist.z + fist.target.momz + fist.height)
		if (isEnemyVictim(victim))
		or (isPlayerVictim(victim))
			if ((CV_FindVar("friendlyfire").value == 0) and not (isInBattle(fist.target.player)))
				return
			end
			P_DamageMobj(victim, fist, fist.target)

			S_StartSound(fist, sfx_bsnipe)
			S_StartSound(fist, sfx_pstop)
			
			P_InstaThrust(victim, fist.target.angle, 32*FRACUNIT + fist.target.player.speed)
			victim.momz = fist.target.momz/3 + 4*FRACUNIT
        end
    end
end, MT_THEFIST)

/*addHook("HurtMsg", function(player, inflictor)
	if (player.mo) and (player.mo.valid)
	and (inflictor.valid) and (inflictor.target.player)
		local targetStatus = "hit"
		if (player.playerstate == PST_DEAD) targetStatus = "killed" end
		print(player.cname.." was "..targetStatus.." by "..inflictor.player.cname.."'s comically large fist.")
	end
end, MT_THEFIST)*/

addHook("ThinkFrame",function()
	if (BattleFuncSet) or not(CBW_Battle and CBW_Battle.SkinVars) then return end
	BattleFuncSet = true
	
	local TheFist = function(mo,doaction,pressedaction)
		local player = mo.player
		player.actiontext = "The Fist"
		player.actionrings = 20
		//Ability start
		if (player.actionstate == 2)
			player.actiontext = "Fist Cancel"
		end
		if doaction == 1 then
			if (player.actionstate == 2)
				player.actionstate = 3
				S_StartSound(player.mo, sfx_spndsh)
				player.freindlygo = TICRATE
				player.freindlydrill = 0
				player.freindlywilldrill = 0
				player.freindlyprevmomz = 0
			end
			player.actionstate = 1
			player.actiontime = 0
			CBW_Battle.PayRings(player)
		end	
		
		if player.actionstate == 1
			CBW_Battle.ApplyCooldown(player,TICRATE*2)
			P_Thrust(player.mo, player.mo.angle, 12*FRACUNIT)
			player.powers[pw_flashing] = 8
			player.actiontime = 0
			player.actionstate = 0
			player.drawangle = player.mo.angle
			local an = player.mo.angle
			local th = P_SpawnMobj(player.mo.x + (20*cos(an)), player.mo.y + (20*sin(an)), player.mo.z+player.mo.height/2, MT_THEFIST)
			th.scale = FRACUNIT/8
			th.color = player.mo.color
			th.target = player.mo

			local speed = 70*FRACUNIT
			if player.mo.eflags & MFE_VERTICALFLIP then
				th.flags2 = $1|MF2_OBJECTFLIP
				th.z = $1 - th.height - player.mo.height/3
			end
			th.angle = an
			th.timers = 0
		end
		
		if player.actionstate == 2
			if (player.freindlywilldrill == 1)
				player.pflags = $|PF_JUMPED
			else
				player.actionstate = 0
			end
		end
	end
	
	CBW_Battle.SkinVars["freindlyfellow"] = {
		weight = 75,
		shields = 1, 
		special = TheFist,
		guard_frame = 2,
		sprites = {}
	}
end)

addHook("MobjThinker", function(fish)
	if not (fish.target) return end
	local mo = fish.target
	fish.target.player.drawangle = fish.target.angle
	fish.timers = $ + 10
	fish.scale = (fish.timers/10)*mo.scale
	if (fish.scale > (3*mo.scale/2))
		fish.scale = (3*mo.scale/2)
	end

	P_MoveOrigin(fish, 
		mo.x + ( max(0,fish.target.player.cmd.forwardmove/2) + 20 + fish.timers/3 ) * cos( mo.angle + fish.target.player.cmd.sidemove/10*ANG1 ),
		mo.y + ( max(0,fish.target.player.cmd.forwardmove/2) + 20 + fish.timers/3 ) * sin( mo.angle + fish.target.player.cmd.sidemove/10*ANG1 ),
		mo.z + mo.height/2
	)
	fish.angle = mo.angle
	if (fish.timers >= 220)
	and (mo.player.freindlygo >= 0)
		P_SpawnGhostMobj(fish)
		P_KillMobj(fish)
		//print("man im dead")
	end
end, MT_THEFIST)